@@ -8,7 +8,7 @@ __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import time, textwrap, os
 from threading import Thread
-from future_builtins import map
+from polyglot.builtins import map
 from operator import itemgetter
 from functools import partial
 from collections import defaultdict
@@ -29,23 +29,26 @@ from calibre.constants import DEBUG
 from calibre.ebooks.oeb.polish.report import (
     gather_data, CSSEntry, CSSFileMatch, MatchLocation, ClassEntry,
     ClassFileMatch, ClassElement, CSSRule, LinkLocation)
-from calibre.gui2 import error_dialog, question_dialog, choose_save_file, open_url
+from calibre.gui2 import error_dialog, question_dialog, choose_save_file, open_url, secure_web_page
 from calibre.gui2.tweak_book import current_container, tprefs, dictionaries
 from calibre.gui2.tweak_book.widgets import Dialog
 from calibre.gui2.progress_indicator import ProgressIndicator
-from calibre.utils.icu import primary_contains, numeric_sort_key, character_name_from_code
+from calibre.utils.icu import primary_contains, numeric_sort_key
+from calibre.utils.unicode_names import character_name_from_code
 from calibre.utils.localization import calibre_langcode_to_name, canonicalize_lang
 
 # Utils {{{
 
 ROOT = QModelIndex()
 
+
 def read_state(name, default=None):
     data = tprefs.get('reports-ui-state')
     if data is None:
         tprefs['reports-ui-state'] = data = {}
     return data.get(name, default)
 
+
 def save_state(name, val):
     data = tprefs.get('reports-ui-state')
     if isinstance(val, QByteArray):
@@ -54,8 +57,10 @@ def save_state(name, val):
         tprefs['reports-ui-state'] = data = {}
     data[name] = val
 
+
 SORT_ROLE = Qt.UserRole + 1
 
+
 class ProxyModel(QSortFilterProxyModel):
 
     def __init__(self, parent=None):
@@ -81,6 +86,7 @@ class ProxyModel(QSortFilterProxyModel):
             return section + 1
         return QSortFilterProxyModel.headerData(self, section, orientation, role)
 
+
 class FileCollection(QAbstractTableModel):
 
     COLUMN_HEADERS = ()
@@ -107,9 +113,10 @@ class FileCollection(QAbstractTableModel):
     def location(self, index):
         try:
             return self.files[index.row()].name
-        except IndexError:
+        except (IndexError, AttributeError):
             pass
 
+
 class FilesView(QTableView):
 
     double_clicked = pyqtSignal(object)
@@ -213,6 +220,7 @@ class FilesView(QTableView):
 
 # Files {{{
 
+
 class FilesModel(FileCollection):
 
     COLUMN_HEADERS = (_('Folder'), _('Name'), _('Size (KB)'), _('Type'))
@@ -262,6 +270,7 @@ class FilesModel(FileCollection):
             if col == 3:
                 return self.CATEGORY_NAMES.get(entry.category)
 
+
 class FilesWidget(QWidget):
 
     edit_requested = pyqtSignal(object)
@@ -274,6 +283,7 @@ class FilesWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = FilesModel(self)
         self.files = f = FilesView(m, self)
         self.to_csv = f.to_csv
@@ -307,6 +317,7 @@ class FilesWidget(QWidget):
 
 # Jump {{{
 
+
 def jump_to_location(loc):
     from calibre.gui2.tweak_book.boss import get_boss
     boss = get_boss()
@@ -327,6 +338,7 @@ def jump_to_location(loc):
         if loc.text_on_line is not None:
             editor.find(regex.compile(regex.escape(loc.text_on_line)))
 
+
 class Jump(object):
 
     def __init__(self):
@@ -341,10 +353,12 @@ class Jump(object):
             loc = locations[self.pos_map[key]]
             jump_to_location(loc)
 
+
 jump = Jump()  # }}}
 
 # Images {{{
 
+
 class ImagesDelegate(QStyledItemDelegate):
 
     MARGIN = 5
@@ -373,25 +387,32 @@ class ImagesDelegate(QStyledItemDelegate):
         k = (th, entry.name)
         pmap = self.cache.get(k)
         if pmap is None:
-            pmap = self.cache[k] = self.pixmap(th, entry)
+            try:
+                dpr = painter.device().devicePixelRatioF()
+            except AttributeError:
+                dpr = painter.device().devicePixelRatio()
+            pmap = self.cache[k] = self.pixmap(th, entry, dpr)
         if pmap.isNull():
             bottom = option.rect.top()
         else:
             m = 2 * self.MARGIN
-            x = option.rect.left() + (option.rect.width() - m - pmap.width()) // 2
+            x = option.rect.left() + (option.rect.width() - m - int(pmap.width()/pmap.devicePixelRatio())) // 2
             painter.drawPixmap(x, option.rect.top() + self.MARGIN, pmap)
-            bottom = m + pmap.height() + option.rect.top()
+            bottom = m + int(pmap.height() / pmap.devicePixelRatio()) + option.rect.top()
         rect = QRect(option.rect.left(), bottom, option.rect.width(), option.rect.bottom() - bottom)
         if option.state & QStyle.State_Selected:
             painter.setPen(self.parent().palette().color(QPalette.HighlightedText))
         painter.drawText(rect, Qt.AlignHCenter | Qt.AlignVCenter, entry.basename)
         painter.restore()
 
-    def pixmap(self, thumbnail_height, entry):
+    def pixmap(self, thumbnail_height, entry, dpr):
         pmap = QPixmap(current_container().name_to_abspath(entry.name)) if entry.width > 0 and entry.height > 0 else QPixmap()
-        scaled, width, height = fit_image(entry.width, entry.height, thumbnail_height, thumbnail_height)
-        if scaled and not pmap.isNull():
-            pmap = pmap.scaled(width, height, transformMode=Qt.SmoothTransformation)
+        if not pmap.isNull():
+            pmap.setDevicePixelRatio(dpr)
+            scaled, width, height = fit_image(entry.width, entry.height, thumbnail_height, thumbnail_height)
+            if scaled:
+                pmap = pmap.scaled(int(dpr * width), int(dpr * height), transformMode=Qt.SmoothTransformation)
+                pmap.setDevicePixelRatio(dpr)
         return pmap
 
 
@@ -452,6 +473,7 @@ class ImagesWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = ImagesModel(self)
         self.files = f = FilesView(m, self)
         self.to_csv = f.to_csv
@@ -489,9 +511,10 @@ class ImagesWidget(QWidget):
 
 # Links {{{
 
+
 class LinksModel(FileCollection):
 
-    COLUMN_HEADERS = ['✓ ', _('Source'), _('Source text'), _('Target'), _('Anchor'), _('Target text')]
+    COLUMN_HEADERS = ['✓', _('Source'), _('Source text'), _('Target'), _('Anchor'), _('Target text')]
 
     def __init__(self, parent=None):
         FileCollection.__init__(self, parent)
@@ -521,7 +544,7 @@ class LinksModel(FileCollection):
             except IndexError:
                 return None
             if col == 0:
-                return {True:'✓ ', False:'✗'}.get(link.ok)
+                return {True:'✓', False:'✗'}.get(link.ok)
             if col == 1:
                 return link.location.name
             if col == 2:
@@ -553,11 +576,13 @@ class LinksModel(FileCollection):
             except IndexError:
                 pass
 
+
 class WebView(QWebView):
 
     def sizeHint(self):
         return QSize(600, 200)
 
+
 class LinksWidget(QWidget):
 
     def __init__(self, parent=None):
@@ -569,6 +594,7 @@ class LinksWidget(QWidget):
         self.splitter = s = QSplitter(Qt.Vertical, self)
         l.addWidget(s)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = LinksModel(self)
         self.links = f = FilesView(m, self)
         f.DELETE_POSSIBLE = False
@@ -578,6 +604,9 @@ class LinksWidget(QWidget):
         s.addWidget(f)
         self.links.restore_table('links-table', sort_column=1)
         self.view = WebView(self)
+        secure_web_page(self.view.page())
+        self.setContextMenuPolicy(Qt.NoContextMenu)
+        self.view.setContextMenuPolicy(Qt.NoContextMenu)
         s.addWidget(self.view)
         self.ignore_current_change = False
         self.current_url = None
@@ -642,6 +671,7 @@ class LinksWidget(QWidget):
 
 # Words {{{
 
+
 class WordsModel(FileCollection):
 
     COLUMN_HEADERS = (_('Word'), _('Language'), _('Times used'))
@@ -653,11 +683,12 @@ class WordsModel(FileCollection):
         self.total_size = len({entry.locale for entry in self.files})
         psk = numeric_sort_key
         lsk_cache = {}
+
         def locale_sort_key(loc):
             try:
                 return lsk_cache[loc]
             except KeyError:
-                lsk_cache[loc] = (psk(calibre_langcode_to_name(canonicalize_lang(loc[0]))), psk(loc[1] or ''))
+                lsk_cache[loc] = psk(calibre_langcode_to_name(canonicalize_lang(loc[0])) + (loc[1] or ''))
             return lsk_cache[loc]
 
         self.sort_keys = tuple((psk(entry.word), locale_sort_key(entry.locale), len(entry.usage)) for entry in self.files)
@@ -693,6 +724,7 @@ class WordsModel(FileCollection):
     def location(self, index):
         return None
 
+
 class WordsWidget(QWidget):
 
     def __init__(self, parent=None):
@@ -702,6 +734,7 @@ class WordsWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = WordsModel(self)
         self.words = f = FilesView(m, self)
         self.to_csv = f.to_csv
@@ -735,6 +768,7 @@ class WordsWidget(QWidget):
 
 # Characters {{{
 
+
 class CharsModel(FileCollection):
 
     COLUMN_HEADERS = (_('Character'), _('Name'), _('Codepoint'), _('Times used'))
@@ -745,11 +779,13 @@ class CharsModel(FileCollection):
         self.files = data['chars']
         self.all_chars = tuple(entry.char for entry in self.files)
         psk = numeric_sort_key
-        self.sort_keys = tuple((psk(entry.char), None, entry.codepoint, len(entry.usage)) for entry in self.files)
+        self.sort_keys = tuple((psk(entry.char), None, entry.codepoint, entry.count) for entry in self.files)
         self.endResetModel()
 
     def data(self, index, role=Qt.DisplayRole):
         if role == SORT_ROLE:
+            if index.column() == 1:
+                return self.data(index)
             try:
                 return self.sort_keys[index.row()][index.column()]
             except IndexError:
@@ -777,6 +813,7 @@ class CharsModel(FileCollection):
     def location(self, index):
         return None
 
+
 class CharsWidget(QWidget):
 
     def __init__(self, parent=None):
@@ -786,6 +823,7 @@ class CharsWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = CharsModel(self)
         self.chars = f = FilesView(m, self)
         self.to_csv = f.to_csv
@@ -837,7 +875,7 @@ class CharsWidget(QWidget):
             ed = boss.edit_file_requested(file_name)
             if ed is None:
                 return
-            if ed.editor.find(pat, complete=not from_cursor):
+            if ed.editor.find_text(pat, complete=not from_cursor):
                 boss.show_editor(file_name)
                 return True
         return False
@@ -846,6 +884,7 @@ class CharsWidget(QWidget):
 
 # CSS {{{
 
+
 class CSSRulesModel(QAbstractItemModel):
 
     def __init__(self, parent):
@@ -948,6 +987,7 @@ class CSSRulesModel(QAbstractItemModel):
         self.build_maps()
         self.endResetModel()
 
+
 class CSSProxyModel(QSortFilterProxyModel):
 
     def __init__(self, parent=None):
@@ -968,6 +1008,7 @@ class CSSProxyModel(QSortFilterProxyModel):
             return True
         return primary_contains(self._filter_text, entry.rule.selector)
 
+
 class CSSWidget(QWidget):
 
     SETTING_PREFIX = 'css-'
@@ -988,6 +1029,7 @@ class CSSWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = self.MODEL(self)
         self.proxy = p = self.PROXY(self)
         p.setSourceModel(m)
@@ -1023,6 +1065,7 @@ class CSSWidget(QWidget):
     def sort_order(self):
         def fget(self):
             return [Qt.AscendingOrder, Qt.DescendingOrder][self._sort_order.currentIndex()]
+
         def fset(self, val):
             self._sort_order.setCurrentIndex({Qt.AscendingOrder:0}.get(val, 1))
         return property(fget=fget, fset=fset)
@@ -1091,6 +1134,7 @@ class CSSWidget(QWidget):
 
 # Classes {{{
 
+
 class ClassesModel(CSSRulesModel):
 
     def __init__(self, parent):
@@ -1168,6 +1212,7 @@ class ClassesModel(CSSRulesModel):
         self.build_maps()
         self.endResetModel()
 
+
 class ClassProxyModel(CSSProxyModel):
 
     def filterAcceptsRow(self, row, parent):
@@ -1179,6 +1224,7 @@ class ClassProxyModel(CSSProxyModel):
             return True
         return primary_contains(self._filter_text, entry.cls)
 
+
 class ClassesWidget(CSSWidget):
 
     SETTING_PREFIX = 'classes-'
@@ -1222,6 +1268,8 @@ class ClassesWidget(CSSWidget):
 # }}}
 
 # Wrapper UI {{{
+
+
 class ReportsWidget(QWidget):
 
     edit_requested = pyqtSignal(object)
@@ -1257,11 +1305,11 @@ class ReportsWidget(QWidget):
 
         self.css = c = CSSWidget(self)
         s.addWidget(c)
-        QListWidgetItem(_('Style Rules'), r)
+        QListWidgetItem(_('Style rules'), r)
 
         self.css = c = ClassesWidget(self)
         s.addWidget(c)
-        QListWidgetItem(_('Style Classes'), r)
+        QListWidgetItem(_('Style classes'), r)
 
         self.chars = c = CharsWidget(self)
         s.addWidget(c)
@@ -1311,6 +1359,7 @@ class ReportsWidget(QWidget):
             with open(fname, 'wb') as f:
                 f.write(data)
 
+
 class Reports(Dialog):
 
     data_gathered = pyqtSignal(object, object)
@@ -1404,6 +1453,7 @@ class Reports(Dialog):
         Dialog.reject(self)
 # }}}
 
+
 if __name__ == '__main__':
     from calibre.gui2 import Application
     import sys
